from unittest.mock import patch

import pytest

from gso.products import EdgePort
from gso.utils.types.interfaces import PhysicalPortCapacity
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    extract_state,
    run_workflow,
)
from test.workflows.iptrunk.test_create_iptrunk import MockedNetboxClient


@pytest.fixture()
def input_form_wizard_data(request, faker, edge_port_subscription_factory, partner_factory):
    subscription_id = edge_port_subscription_factory()

    return [
        {"subscription_id": subscription_id},
        {
            "tt_number": faker.tt_number(),
            "geant_ga_id": faker.geant_gid(),
            "member_speed": PhysicalPortCapacity.FOUR_HUNDRED_GIGABIT_PER_SECOND,
            "number_of_members": 1,
        },
        {
            "description": faker.sentence(),
            "ae_members": [
                {
                    "interface_name": "Interface1",
                    "interface_description": faker.sentence(),
                }
            ],
        },
    ]


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
@patch("gso.services.netbox_client.NetboxClient.get_available_interfaces")
@patch("gso.services.netbox_client.NetboxClient.attach_interface_to_lag")
@patch("gso.services.netbox_client.NetboxClient.reserve_interface")
@patch("gso.services.netbox_client.NetboxClient.allocate_interface")
@patch("gso.services.netbox_client.NetboxClient.free_interface")
@patch("gso.services.netbox_client.NetboxClient.detach_interfaces_from_lag")
@patch("gso.services.netbox_client.NetboxClient.get_interface_by_name_and_device")
def test_modify_edge_port_with_changing_capacity(
    mocked_get_interface_by_name_and_device,
    mocked_detach_interfaces_from_lag,
    mocked_free_interface,
    mocked_allocate_interface,
    mocked_reserve_interface,
    mocked_attach_interface_to_lag,
    mocked_get_available_interfaces,
    mocked_execute_playbook,
    input_form_wizard_data,
    faker,
    data_config_filename,
):
    #  Set up mock return values
    mocked_netbox = MockedNetboxClient()
    mocked_get_available_interfaces.return_value = mocked_netbox.get_available_interfaces()
    mocked_attach_interface_to_lag.return_value = mocked_netbox.attach_interface_to_lag()
    mocked_reserve_interface.return_value = mocked_netbox.reserve_interface()
    mocked_allocate_interface.return_value = mocked_netbox.allocate_interface()
    mocked_free_interface.return_value = mocked_netbox.free_interface()
    mocked_detach_interfaces_from_lag.return_value = mocked_netbox.detach_interfaces_from_lag()
    mocked_get_interface_by_name_and_device.side_effect = mocked_netbox.get_interface_by_name_and_device

    #  Run workflow
    result, process_stat, step_log = run_workflow("modify_edge_port", input_form_wizard_data)

    for _ in range(2):
        result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)
    # Validate the final state and subscription data
    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = EdgePort.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert mocked_execute_playbook.call_count == 2

    # The number of members have been changed from 2 to 1
    assert mocked_reserve_interface.call_count == 1
    assert mocked_attach_interface_to_lag.call_count == 1
    assert mocked_free_interface.call_count == 2
    assert mocked_detach_interfaces_from_lag.call_count == 1
    assert subscription.edge_port.geant_ga_id == input_form_wizard_data[1]["geant_ga_id"]
    assert len(subscription.edge_port.edge_port_ae_members) == 1


@pytest.fixture()
def input_form_wizard_without_changing_capacity(request, faker, edge_port_subscription_factory, partner_factory):
    subscription_id = edge_port_subscription_factory()
    subscription = EdgePort.from_subscription(subscription_id)

    return [
        {"subscription_id": subscription_id},
        {"tt_number": faker.tt_number(), "geant_ga_id": faker.geant_gid()},
        {
            "description": faker.sentence(),
            "ae_members": [
                {
                    "interface_name": interface.interface_name,
                    "interface_description": interface.interface_description,
                }
                for interface in subscription.edge_port.edge_port_ae_members
            ],
        },
    ]


@pytest.mark.workflow()
@patch("gso.services.lso_client._send_request")
@patch("gso.services.netbox_client.NetboxClient.get_available_interfaces")
@patch("gso.services.netbox_client.NetboxClient.attach_interface_to_lag")
@patch("gso.services.netbox_client.NetboxClient.reserve_interface")
@patch("gso.services.netbox_client.NetboxClient.allocate_interface")
@patch("gso.services.netbox_client.NetboxClient.free_interface")
@patch("gso.services.netbox_client.NetboxClient.detach_interfaces_from_lag")
@patch("gso.services.netbox_client.NetboxClient.get_interface_by_name_and_device")
def test_modify_edge_port_without_changing_capacity(
    mocked_get_interface_by_name_and_device,
    mocked_detach_interfaces_from_lag,
    mocked_free_interface,
    mocked_allocate_interface,
    mocked_reserve_interface,
    mocked_attach_interface_to_lag,
    mocked_get_available_interfaces,
    mocked_execute_playbook,
    input_form_wizard_without_changing_capacity,
    faker,
    data_config_filename,
):
    #  Set up mock return values
    mocked_netbox = MockedNetboxClient()
    mocked_get_available_interfaces.return_value = mocked_netbox.get_available_interfaces()
    mocked_attach_interface_to_lag.return_value = mocked_netbox.attach_interface_to_lag()
    mocked_reserve_interface.return_value = mocked_netbox.reserve_interface()
    mocked_allocate_interface.return_value = mocked_netbox.allocate_interface()
    mocked_free_interface.return_value = mocked_netbox.free_interface()
    mocked_detach_interfaces_from_lag.return_value = mocked_netbox.detach_interfaces_from_lag()
    mocked_get_interface_by_name_and_device.side_effect = mocked_netbox.get_interface_by_name_and_device

    #  Run workflow
    result, _, _ = run_workflow("modify_edge_port", input_form_wizard_without_changing_capacity)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = EdgePort.from_subscription(subscription_id)

    assert subscription.status == "active"

    # The capacity has not been changed so the following methods should not be called
    assert mocked_execute_playbook.call_count == 0
    assert mocked_reserve_interface.call_count == 0
    assert mocked_attach_interface_to_lag.call_count == 0
    assert mocked_free_interface.call_count == 0
    assert mocked_detach_interfaces_from_lag.call_count == 0

    assert subscription.edge_port.geant_ga_id == input_form_wizard_without_changing_capacity[1]["geant_ga_id"]
    assert len(subscription.edge_port.edge_port_ae_members) == 2
    assert subscription.edge_port.edge_port_description == input_form_wizard_without_changing_capacity[2]["description"]
