import pytest

from gso.products.product_blocks.bgp_session import IPFamily
from gso.products.product_types.nren_l3_core_service import NRENL3CoreService
from gso.utils.shared_enums import APType
from test.workflows import extract_state, run_workflow


@pytest.mark.workflow()
def test_modify_geant_ip_remove_edge_port_success(geant_ip_subscription_factory):
    subscription_id = geant_ip_subscription_factory()
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    access_port = subscription.geant_ip.geant_ip_ap_list[0]
    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "geant_ip_ep": str(access_port.geant_ip_sbp.edge_port.owner_subscription_id),
                    "nren_ap_type": APType.LOAD_BALANCED,
                }
            ]  # The factory generates a subscription with two Access Ports, this will remove the second one.
        },
        {},
    ]

    result, _, _ = run_workflow("modify_geant_ip", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.geant_ip.geant_ip_ap_list) == 1
    assert subscription.geant_ip.geant_ip_ap_list[0].nren_ap_type == APType.LOAD_BALANCED


@pytest.mark.workflow()
def test_modify_geant_ip_add_new_edge_port_success(
    geant_ip_subscription_factory, edge_port_subscription_factory, partner_factory, faker
):
    partner = partner_factory()
    new_edge_port = edge_port_subscription_factory(partner=partner)
    subscription_id = geant_ip_subscription_factory(partner=partner)
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "geant_ip_ep": str(port.geant_ip_sbp.edge_port.owner_subscription_id),
                    "nren_ap_type": port.nren_ap_type,
                }
                for port in subscription.geant_ip.geant_ip_ap_list
            ]
            + [
                {
                    "geant_ip_ep": str(new_edge_port),
                    "nren_ap_type": APType.BACKUP,
                }
            ]
        },
        {},  # The existing SBPs are unchanged
        {},
        {  # Adding configuration for the new SBP
            "geant_sid": faker.geant_sid(),
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "v4_bgp_peer": {
                "authentication_key": faker.password(),
                "peer_address": faker.ipv4(),
            },
            "v6_bgp_peer": {
                "authentication_key": faker.password(),
                "peer_address": faker.ipv6(),
            },
        },
    ]

    result, _, _ = run_workflow("modify_geant_ip", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.geant_ip.geant_ip_ap_list) == 3


@pytest.mark.workflow()
def test_modify_geant_ip_modify_edge_port_success(faker, geant_ip_subscription_factory):
    subscription_id = geant_ip_subscription_factory()
    subscription = NRENL3CoreService.from_subscription(subscription_id)
    new_sbp_data = [
        {
            "geant_sid": faker.geant_sid(),
            "is_tagged": True,
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "custom_firewall_filters": True,
            "v4_bgp_peer": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv4(),
                "add_v4_multicast": True,
            },
            "v6_bgp_peer": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv6(),
                "add_v6_multicast": True,
            },
        },
        {
            "geant_sid": faker.geant_sid(),
            "is_tagged": True,
            "vlan_id": faker.vlan_id(),
            "ipv4_address": faker.ipv4(),
            "ipv6_address": faker.ipv6(),
            "custom_firewall_filters": True,
            "v4_bgp_peer": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv4(),
                "add_v4_multicast": True,
            },
            "v6_bgp_peer": {
                "bfd_enabled": True,
                "bfd_interval": faker.pyint(),
                "bfd_multiplier": faker.pyint(),
                "has_custom_policies": True,
                "authentication_key": faker.password(),
                "multipath_enabled": True,
                "send_default_route": True,
                "is_passive": True,
                "peer_address": faker.ipv6(),
                "add_v6_multicast": True,
            },
        },
    ]

    input_form_data = [
        {"subscription_id": subscription_id},
        {
            "access_ports": [
                {
                    "geant_ip_ep": str(port.geant_ip_sbp.edge_port.owner_subscription_id),
                    "nren_ap_type": port.nren_ap_type,
                }
                for port in subscription.geant_ip.geant_ip_ap_list
            ]
        },
        {**new_sbp_data[0]},
        {**new_sbp_data[1]},
    ]

    result, _, _ = run_workflow("modify_geant_ip", input_form_data)

    state = extract_state(result)
    subscription = NRENL3CoreService.from_subscription(state["subscription_id"])
    assert len(subscription.geant_ip.geant_ip_ap_list) == 2

    for i in range(2):
        assert subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.geant_sid == new_sbp_data[i]["geant_sid"]
        assert subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.is_tagged == new_sbp_data[i]["is_tagged"]
        assert subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.vlan_id == new_sbp_data[i]["vlan_id"]
        assert (
            str(subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.ipv4_address) == new_sbp_data[i]["ipv4_address"]
        )
        assert (
            str(subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.ipv6_address) == new_sbp_data[i]["ipv6_address"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.custom_firewall_filters
            == new_sbp_data[i]["custom_firewall_filters"]
        )

        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].bfd_enabled
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_enabled"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].bfd_interval
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_interval"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].bfd_multiplier
            == new_sbp_data[i]["v4_bgp_peer"]["bfd_multiplier"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].has_custom_policies
            == new_sbp_data[i]["v4_bgp_peer"]["has_custom_policies"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].authentication_key
            == new_sbp_data[i]["v4_bgp_peer"]["authentication_key"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].multipath_enabled
            == new_sbp_data[i]["v4_bgp_peer"]["multipath_enabled"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].send_default_route
            == new_sbp_data[i]["v4_bgp_peer"]["send_default_route"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].is_passive
            == new_sbp_data[i]["v4_bgp_peer"]["is_passive"]
        )
        assert (
            str(subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].peer_address)
            == new_sbp_data[i]["v4_bgp_peer"]["peer_address"]
        )
        assert (
            bool(
                IPFamily.V4MULTICAST
                in subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[0].families
            )
            == new_sbp_data[i]["v4_bgp_peer"]["add_v4_multicast"]
        )

        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].bfd_enabled
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_enabled"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].bfd_interval
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_interval"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].bfd_multiplier
            == new_sbp_data[i]["v6_bgp_peer"]["bfd_multiplier"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].has_custom_policies
            == new_sbp_data[i]["v6_bgp_peer"]["has_custom_policies"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].authentication_key
            == new_sbp_data[i]["v6_bgp_peer"]["authentication_key"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].multipath_enabled
            == new_sbp_data[i]["v6_bgp_peer"]["multipath_enabled"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].send_default_route
            == new_sbp_data[i]["v6_bgp_peer"]["send_default_route"]
        )
        assert (
            subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].is_passive
            == new_sbp_data[i]["v6_bgp_peer"]["is_passive"]
        )
        assert (
            str(subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].peer_address)
            == new_sbp_data[i]["v6_bgp_peer"]["peer_address"]
        )
        assert (
            bool(
                IPFamily.V6MULTICAST
                in subscription.geant_ip.geant_ip_ap_list[i].geant_ip_sbp.sbp_bgp_session_list[1].families
            )
            == new_sbp_data[i]["v6_bgp_peer"]["add_v6_multicast"]
        )
